# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-23 08:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0020_auto_20190522_1717'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='sn',
            field=models.IntegerField(help_text='\u53ea\u9700\u586b\u5199\u6570\u5b57', null=True, unique=True, verbose_name='\u7edf\u4e00\u7f26\u5ba2\u53f7'),
        ),
    ]
