# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-31 06:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0012_auto_20181031_1328'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='is_leave',
            field=models.BooleanField(default=False, verbose_name='\u662f\u5426\u8bf7\u5047\u4e2d'),
        ),
    ]
