# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-16 08:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0004_auto_20181016_1500'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='expire_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='\u7ed3\u8bfe\u65e5\u671f'),
        ),
        migrations.AddField(
            model_name='userprofile',
            name='start_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='\u5f00\u8bfe\u65e5\u671f'),
        ),
    ]
