# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-05-16 10:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='userclass',
            options={'ordering': ['-created_at'], 'permissions': (('user.class.view_all', 'view all classes'), ('user.class.view_manage', 'view manage classes'), ('user.class.view_teach', 'view teach classes'))},
        ),
    ]
