# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-26 09:20
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_user', '0005_auto_20181016_1619'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserLeaveRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.IntegerField(choices=[(1, '\u8bf7\u5047'), (2, '\u9500\u5047'), (3, '\u5ef6\u671f')], default=1)),
                ('start', models.DateTimeField(blank=True, null=True, verbose_name='\u5f00\u59cb\u65e5\u671f')),
                ('end', models.DateTimeField(blank=True, null=True, verbose_name='\u7ed3\u675f\u65e5\u671f')),
                ('old_expire', models.DateTimeField(blank=True, verbose_name='\u539f\u7ed3\u8bfe\u65e5\u671f')),
                ('new_expire', models.DateTimeField(blank=True, verbose_name='\u65b0\u7ed3\u8bfe\u65e5\u671f')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('is_check', models.BooleanField(default=False, verbose_name='\u901a\u8fc7\u5ba1\u6838')),
                ('check_at', models.DateTimeField(null=True)),
                ('check_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leave_user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created_at'],
                'db_table': 'bee_django_user_leave_record',
                'permissions': (('change_check', '\u53ef\u4ee5\u5ba1\u6838\u7528\u6237\u7684\u8bf7\u5047'),),
            },
        ),
    ]
