# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-22 08:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_user', '0018_auto_20190522_1635'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usersn',
            name='end',
            field=models.IntegerField(unique=True, verbose_name='\u6b62'),
        ),
    ]
