# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-10-30 05:16
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bee_django_user', '0007_auto_20181026_1751'),
    ]

    operations = [
        migrations.AddField(
            model_name='userleaverecord',
            name='created_by',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='create_user', to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='userleaverecord',
            name='check_by',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='check_user', to=settings.AUTH_USER_MODEL),
        ),
    ]
