{% extends 'bee_django_user/base.html' %}
{% load bootstrap3 %}
{% load static %}

{% block content %}

    {% include 'bee_django_user/logo.html' %}
    {% include 'bee_django_user/nav.html' %}
    {% include 'bee_django_user/messages.html' %}
    <h4 class="text-center">重置[{{ user.username }}]{{ user.first_name }}的密码</h4>
    <div style="margin:20px;">
        <form method="post">
            {% csrf_token %}
            {% bootstrap_form form %}
            <input type="submit" value="重置">
            <!-- <a href="#">课程卡注册</a> -->
        </form>
    </div>
{% endblock content %}